(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2005-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: value_type_def.mli,v 1.12 2017/05/31 15:54:22 fclement Exp $ *)

(** {6 The definition of typed-values types} *)

(** {6 The private definition part of typed-values types} *)

(** This is the private part of the Caml type that defines the types of the
basic entities used inside communications, the so-called typed-values. *)

type scalar_type = private
| Bool_type
| String_type
| Int_type | Native_int_type | Int32_type | Int64_type
| Float_type | Float_bin_type
| Couple_type of scalar_type * scalar_type
| Triple_type of scalar_type * scalar_type * scalar_type
;;

type value_type = private
| Scalar_type of scalar_type
| Vector_type of scalar_type
| Matrix_type of scalar_type
;;

(** {6 The public construction functions for [scalar_type]} *)

(** [scalar_type] is the Caml private type that defines the types of
  scalars. *)

val bool_type : scalar_type;;
val string_type : scalar_type;;
val int_type : scalar_type;;
val native_int_type : scalar_type;;
val int32_type : scalar_type;;
val int64_type : scalar_type;;
val float_type : scalar_type;;
val float_bin_type : scalar_type;;

val couple_type : scalar_type * scalar_type -> scalar_type;;

val triple_type : scalar_type * scalar_type * scalar_type -> scalar_type
;;

(** {6 The public construction functions for [value_type]} *)

(** [value_type] is the Caml private type that defines the types of
  typed-values. *)

val scalar_type : scalar_type -> value_type;;
val vector_type : scalar_type -> value_type;;
val matrix_type : scalar_type -> value_type;;
