(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2005-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: write_communication.ml,v 1.20 2017/05/31 15:54:22 fclement Exp $ *)

module Nat = Unatural;;
module A = Uarray;;

open Communication;;

let qual = Upervasives.qualify "Write_communication";;

let debug fun_name = Options_pio.pf.Udebug.debug (Some (qual fun_name));;

let w_values ob vals =
  debug "w_values" "start writing";
  A.iter (Write_typed_value.write_typed_value ob) vals;
  debug "w_values" "finish writing";
;;

let d_com_msg = function
  | Pio_Phrase vals ->
      let n = A.length vals in
      Printf.sprintf "Phrase of length %d" (Nat.to_int n)
  | Pio_Task (task_name, vals) ->
      let n = A.length vals in
      Printf.sprintf "Task %S of length %d" task_name (Nat.to_int n)
  | Pio_Result vals ->
      let n = A.length vals in
      Printf.sprintf "Result of length %d" (Nat.to_int n)
  | Pio_Error vals ->
      let n = A.length vals in
      Printf.sprintf "Error of length %d" (Nat.to_int n)
  | Pio_Service (service_name, vals) ->
      let n = A.length vals in
      Printf.sprintf "Service %S of length %d"
        (string_of_service_name service_name) (Nat.to_int n)
;;

let write_com ob c =
  debug "write_com" (Printf.sprintf "start writing %s" (d_com_msg c));
  begin match c with
  | Pio_Phrase vals ->
      let n = A.length vals in
      Printf.bprintf ob "%%p <%d>\n%a"
        (Nat.to_int n) w_values vals
  | Pio_Task (task_name, vals) ->
      let n = A.length vals in
      Printf.bprintf ob "%%t <%d> %S\n%a"
        (Nat.to_int n) task_name w_values vals
  | Pio_Result vals ->
      let n = A.length vals in
      Printf.bprintf ob "%%r <%d>\n%a"
        (Nat.to_int n) w_values vals
  | Pio_Error vals ->
      let n = A.length vals in
      Printf.bprintf ob "%%e <%d>\n%a"
        (Nat.to_int n) w_values vals
  | Pio_Service (service_name, vals) ->
      let n = A.length vals in
      Printf.bprintf ob "%%s <%d> %S\n%a"
        (Nat.to_int n) (string_of_service_name service_name) w_values vals
  end;
  debug "write_com" "finish writing";
;;

let write_communication ob = Printf.bprintf ob "(\n%a)\n\n" write_com;;
