(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: colormapping_print.ml,v 1.5 2017/05/31 17:27:34 fclement Exp $ *)

(* Colormapping printers. *)

let printer_color_scale ctx ppf = function
  | c_t_a -> Og_printer.printer_Array Colors_print.printer_t ctx ppf c_t_a
;;

let printer_from_float ctx ppf = function
  | f -> Og_printer.printer_Float ctx ppf f

and printer_to_float ctx ppf = function
  | f -> Og_printer.printer_Float ctx ppf f
;;

let printer_mapping ctx ppf = function
  | f_c_f_f_f_t_f_f_c_t ->
    Og_printer.printer_Fun
      printer_color_scale
      (Og_printer.printer_Fun
         printer_from_float
         (Og_printer.printer_Fun
            printer_to_float
            (Og_printer.printer_Fun
               Og_printer.printer_Float
               Colors_print.printer_t)))
      ctx
      ppf
      f_c_f_f_f_t_f_f_c_t
;;

let printer_t ctx ppf = function
  | {
      Colormapping.color_scale = c;
      Colormapping.from_float = f;
      Colormapping.to_float = t;
      Colormapping.mapping = m;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,Colormapping.@,";
    Format.fprintf ppf "%,@[<hv 2>color_scale =";
    printer_color_scale Og_precedence.Ck_record_field ppf c;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>from_float =";
    printer_from_float Og_precedence.Ck_record_field ppf f;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>to_float =";
    printer_to_float Og_precedence.Ck_record_field ppf t;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>mapping =";
    printer_mapping Og_precedence.Ck_record_field ppf m;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_num_steps ctx ppf = function
  | i -> Og_printer.printer_Int ctx ppf i
;;

let printer_from_color ctx ppf = function
  | c_t -> Colors_print.printer_t ctx ppf c_t

and printer_to_color ctx ppf = function
  | c_t -> Colors_print.printer_t ctx ppf c_t
;;

let printer_color_scaler ctx ppf = function
  | f_f_f_t_f_n_f_i_c_t ->
    Og_printer.printer_Fun
      printer_from_color
      (Og_printer.printer_Fun
         printer_to_color
         (Og_printer.printer_Fun
            printer_num_steps
            (Og_printer.printer_Fun
               Og_printer.printer_Int
               Colors_print.printer_t)))
      ctx
      ppf
      f_f_f_t_f_n_f_i_c_t
;;

(* Defining top printers. *)

let print_color_scale ppf = printer_color_scale Og_precedence.Ck_toplevel ppf
;;

let print_from_float ppf = printer_from_float Og_precedence.Ck_toplevel ppf

and print_to_float ppf = printer_to_float Og_precedence.Ck_toplevel ppf
;;

let print_mapping ppf = printer_mapping Og_precedence.Ck_toplevel ppf
;;

let print_t ppf = printer_t Og_precedence.Ck_toplevel ppf
;;

let print_num_steps ppf = printer_num_steps Og_precedence.Ck_toplevel ppf
;;

let print_from_color ppf = printer_from_color Og_precedence.Ck_toplevel ppf

and print_to_color ppf = printer_to_color Og_precedence.Ck_toplevel ppf
;;

let print_color_scaler ppf =
  printer_color_scaler Og_precedence.Ck_toplevel ppf
;;
