(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uhistory_spec.mli,v 1.18 2017/11/07 20:57:39 fclement Exp $ *)

(** {3 History specification} *)

open Uhistory_types;;

module type CONFIGURATION =
sig
  type t;;

  val printer : t Og_types.printer;;
  val print : t Og_types.top_printer;;

  val cold_set : unit -> unit;;
  val warm_set : t -> unit;;
  val get : unit -> t;;
end
;;

module type STATE =
sig
  type t;;
  type constants;;

  val same : t -> t -> bool;;
  val printer : t Og_types.printer;;
  val print : t Og_types.top_printer;;
  val printer_constants : constants Og_types.printer;;
  val print_constants : constants Og_types.top_printer;;

  val init : unit -> t;;
  val init_constants : t -> constants;;
  val final : t -> unit;;
end
;;

module type HISTORY =
sig
  type config;;
  type constants;;
  type state;;

  type t = (config, constants, state) Uhistory_types.t;;

  val make_history_step : state_number -> state -> t -> t;;

  val head_state : t -> state;;
  val head_state_number : t -> state_number;;

  val same_head_state : t -> t -> bool;;

  val printer : t Og_types.printer;;
  val print : t Og_types.top_printer;;
  val printer_head_h_state : t Og_types.printer;;
  val print_head_h_state : t Og_types.top_printer;;

  val initialization : unit -> t;;
  val finalization : t -> unit;;
end
;;

module type MAKER =
  functor (Config : CONFIGURATION) ->
  functor (St : STATE) ->
  HISTORY with
    type state = St.t
    and type constants = St.constants
;;
