(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uvector.ml,v 1.19 2017/11/07 20:57:39 fclement Exp $ *)

(* Shortcuts for the Ulib library. *)

module L = Ulist;;
module A = Uarray;;

(* Fixed dimension vector operations. *)

module Make =
  functor (Dim : Uvector_spec.DIMENSION) ->
struct
  include Dim;;

  module Vt = Uvector_types.Make (Dim);;

  type 'a t = 'a Vt.t;;

  let to_array = Vt.of_vector
  and of_array = Vt.to_vector
  ;;

  external unsafe_of_array : 'a array -> 'a t = "%identity";;

  let get v = A.get (to_array v);;

  let make x = unsafe_of_array (A.make (get_dim ()) x);;
  let init f = unsafe_of_array (A.init (get_dim ()) f);;

  let to_list v = A.to_list (to_array v);;
  let of_list l = of_array (A.of_list l);;
  let iter f v = A.iter f (to_array v);;
  let map f v = unsafe_of_array (A.map f (to_array v));;
  let iter_i f v = A.iter_i f (to_array v);;
  let map_i f v = unsafe_of_array (A.map_i f (to_array v));;
  let iter_rails fv v = A.iter_rails (to_array fv) (to_array v);;
  let fold_left f accu v = A.fold_left f accu (to_array v);;
  let fold_right f v = A.fold_right f (to_array v);;
  let sort cmp v =
    let a = to_array v in
    A.sort cmp a;
    unsafe_of_array a
  ;;

  let fold_left_i f accu v = A.fold_left_i f accu (to_array v);;
  let fold_right_i f v = A.fold_right_i f (to_array v);;
  let iter2 f v1 v2 = A.iter2 f (to_array v1) (to_array v2);;
  let map2 f v1 v2 =
    unsafe_of_array (A.map2 f (to_array v1) (to_array v2))
  ;;
  let iter2_i f v1 v2 = A.iter2_i f (to_array v1) (to_array v2);;
  let map2_i f v1 v2 =
    unsafe_of_array (A.map2_i f (to_array v1) (to_array v2))
  ;;
  let fold_left2 f accu v1 v2 =
    A.fold_left2 f accu (to_array v1) (to_array v2)
  ;;
  let fold_right2 f v1 v2 = A.fold_right2 f (to_array v1) (to_array v2);;
  let iter3 f v1 v2 v3 =
    A.iter3 f (to_array v1) (to_array v2) (to_array v3)
  ;;
  let map3 f v1 v2 v3 =
    unsafe_of_array (A.map3 f (to_array v1) (to_array v2) (to_array v3))
  ;;
  let iter4 f v1 v2 v3 v4 =
    A.iter4 f (to_array v1) (to_array v2) (to_array v3) (to_array v4)
  ;;
  let map4 f v1 v2 v3 v4 =
    unsafe_of_array
      (A.map4 f (to_array v1) (to_array v2) (to_array v3) (to_array v4))
  ;;
  let for_all p v = A.for_all p (to_array v);;
  let for_all2 p v1 v2 = A.for_all2 p (to_array v1) (to_array v2);;
  let exists p v = A.exists p (to_array v);;
  let find p v = A.find p (to_array v);;
  let rev_find_all p v = A.rev_find_all p (to_array v);;

  let max_assoc v =
    let l = A.to_list (to_array v) in
    L.max_assoc l
  ;;

  let printer p_a ctx ppf v =
    Og_printer.printer_Array p_a ctx ppf (to_array v)
  ;;
end
;;
