(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uvector_types.ml,v 1.14 2017/11/07 20:57:39 fclement Exp $ *)

(* Shortcuts for the Ulib library. *)

module P = Upervasives;;
module A = Uarray;;

(* Fixed dimension vector private type. *)

let qual = P.qualify "Uvector";;
let failwith = P.failwith_qual qual;;

(* This functor must be defined before interface only module Vector_spec
 so that it can be used in it to define module type Vector_spec.MAKER.
 Hence, vector_types stuff cannot be incorporated into vector counterparts,
 i.e. Vector_types_spec into Vector_spec, and Vector_types into Vector. *)

module Make =
  functor (Dim : Uvector_types_spec.DIMENSION) ->
struct
  type 'a t = 'a array;;

  let to_vector a =
    if A.length a <> Dim.get_dim () then
      failwith "of_array" "wrong length" else
    a
  ;;

  external of_vector : 'a t -> 'a array = "%identity";;
end
;;
