(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uindex_universe.ml,v 1.12 2017/11/07 20:57:40 fclement Exp $*)

(* Shortcuts for the Ulib library. *)

module P = Upervasives;;
module L1 = Ulist1;;
module A = Uarray;;
module Idx = Uindex;;

(* Shortcuts for the Mapping library. *)

module Iidx = Uindex_index;;
module Inum = Uindex_numbering;;

(* Index numbering operations. *)

let qual = P.qualify "Uindex_universe";;
let invalid_arg = P.invalid_arg_qual qual;;

type 'a element = 'a Inum.key;; (* = 'a Idx.t *)
type 'a index = 'a Inum.value;; (* = 'a element Idx.t *)

type 'a t =
  {
   tables : 'a Inum.t;
   cardinal : P.length;
  }
;;

(* Construction functions. *)

let of_array a =
  let s1h = Inum.make a in
  { tables = s1h; cardinal = Inum.length s1h; }
;;

let of_list1 l1 = of_array (A.of_list1 l1);;

let of_nonempty_range l r =
  if l >= r then invalid_arg "of_nonempty_range" "empty range" else
  of_list1 (L1.of_range Idx.pred l r)
;;

(* Access functions. *)

let cardinal iu = iu.cardinal;;

let mem iu = Inum.mem iu.tables;;

let index iu = Inum.value_of_key iu.tables;;

let element iu = Inum.key_of_value iu.tables;;

let elements iu =
  A.init (cardinal iu) (fun i -> element iu (Iidx.of_nat i))
;;

let equal iu1 iu2 =
  iu1 == iu2 ||
  Inum.equal iu1.tables iu2.tables
;;

(* Iterators. *)

let iter f iu = Inum.iter (fun x idx -> f idx x) iu.tables;;

let sub_iter f iu = Inum.sub_iter (fun x idx -> f idx x) iu.tables;;

let fold_left f accu iu =
  Inum.fold_left (fun accu x idx -> f accu idx x) accu iu.tables
;;
