(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: umatrix_index.ml,v 1.13 2017/11/07 20:57:40 fclement Exp $*)

(* Shortcuts for the Ulib library. *)

module Nat = Unatural;;
module A = Uarray;;
module Idx = Uindex;;

(* Matrix index operations. *)

type nat = Nat.t;;
type nat_couple = nat * nat;;

type element = nat_couple;;
type index = element Idx.t;;

let to_index = Idx.to_t
and of_index = Idx.of_t
;;

let of_nat = Idx.of_nat
and to_nat = Idx.to_nat
;;

let of_int = Idx.of_int
and to_int = Idx.to_int
;;

let _0_ = Idx._0_
and _1_ = Idx._1_
;;

let succ = Idx.succ
and pred = Idx.pred
;;

let uindex_incr r = r := succ !r
and uindex_decr r = r := pred !r
;;

let incr = uindex_incr
and decr = uindex_decr
;;

let array_get a v = A.get a (to_nat v);;
