(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: umatrix_numbering.mli,v 1.12 2017/10/27 13:30:02 fclement Exp $*)

(** {3 Matrix numbering bijections}

 Specialized numbering bijections for numbering matrix slots, i.e. couples of
 natural numbers in a rectangle.
 In this case, a costless bijective numbering function can be defined.
 Type [Umatrix_numbering.t] is an efficient alternative to
 [(Unatural.t * Unatural.t, Unatural.t) Hashtbl.t]. *)

type size = Umatrix_mask.size;;

type key = Umatrix_index.element;;
type value = Umatrix_index.index;;

type t = private
  {
   size : size;
   numbering : key Unumbering.t;
  }
;;

(** {6 Construction functions} *)

val make : size -> t;;

val of_mask : Umatrix_mask.t -> t;;
val make_masked : Umatrix_mask.selection_matrix -> t;;

(** {6 Access functions} *)

val length : t -> Upervasives.length;;
val size : t -> size;;

val mem : t -> key -> bool;;

val value_of_key : t -> key -> value;;

val key_of_value : t -> value -> key;;

(** {6 Iterators} *)

val iter : (key -> value -> unit) -> t -> unit;;

val fold_left : ('accu -> key -> value -> 'accu) -> 'accu -> t -> 'accu;;
