(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: unumbering.ml,v 1.6 2017/11/07 20:57:40 fclement Exp $*)

(* Shortcuts for the Ulib library.*)

module Nat = Unatural;;
module P = Upervasives;;
module Idx = Uindex;;

let _0_ = Nat._0_;;

(* Numbering bijection operations. *)

let qual = P.qualify "Unumbering";;
let invalid_arg = P.invalid_arg_qual qual;;

type length = P.length;;

type 'a value = 'a Idx.t;;
type 'key h_dir = 'key -> 'key value;;
type 'key h_inv = 'key value -> 'key;;

type 'key t =
  {
   length : P.length;
   h_dir : 'key h_dir;
   h_inv : 'key h_inv;
  }
;;

let make len h_dir h_inv = { length = len; h_dir = h_dir; h_inv = h_inv; };;

(* Access functions. *)

let length nb = nb.length;;

let value_of_key nb = nb.h_dir;;

let key_of_value nb = nb.h_inv;;

let equal
    { length = len1; h_inv = h_inv1; _; }
    { length = len2; h_inv = h_inv2; _; } =
  len1 = len2 &&
  let rec loop i =
    Idx.to_nat i = len1 ||
    h_inv1 i = h_inv2 i && loop (Idx.succ i) in
  loop Idx._0_
;;

(* Iterators. *)

let check_sub_hashtbl fun_name nb ofs len =
  let ofs = Idx.to_nat ofs in
  if ofs < _0_ || Nat.add ofs len > length nb then
    invalid_arg fun_name "wrong sub hash table specification"
;;

let sub_fold_left f accu nb ofs len =
  check_sub_hashtbl "sub_fold_left" nb ofs len;
  let lim = Nat.add (Idx.to_nat ofs) len in
  let rec loop accu v =
    if Idx.to_nat v = lim then accu else
    let k = nb.h_inv v in
    let accu = f accu k v in
    loop accu (Idx.succ v) in
  loop accu ofs
;;

let fold_left f accu nb = sub_fold_left f accu nb Idx._0_ (length nb);;

let sub_iter f = sub_fold_left (fun _ k v -> f k v) ();;

let iter f = fold_left (fun _ k v -> f k v) ();;
