(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: optimization_status.ml,v 1.3 2017/11/07 20:57:41 fclement Exp $ *)

(* Shortcuts for the Ulib library. *)

module P = Upervasives;;

(* Shortcuts for the Refinement library. *)

open Optimization_status_types;;
open Optimization_status_types_print;;

(* Optimization status operations. *)

let qual = P.qualify "Optimization_status";;
let invalid_arg = P.invalid_arg_qual qual;;

let scan = function
  | "OS_Bad_initialization" -> OS_Bad_initialization
  | "OS_Convergence" -> OS_Convergence
  | "OS_Degenerated_problem" -> OS_Degenerated_problem
  | "OS_Stagnation" -> OS_Stagnation
  | "OS_Too_many_iterations" -> OS_Too_many_iterations
  | "OS_Too_many_simulations" -> OS_Too_many_simulations
  | s ->
    invalid_arg "scan" (Printf.sprintf "Unknown optimization status %S" s)
;;

let to_string =
  let b = Buffer.create 16 in
  let ppf = Format.formatter_of_buffer b in
  (fun os ->
   Buffer.clear b;
   Format.fprintf ppf "%a%!" print_t os;
   Buffer.contents b)
;;

let sprint ident os = Printf.sprintf "%s = %s" ident (to_string os);;
