(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of OCamlGen, 4.2+dev0 generated code. *)

open Refinement_types_print;;

open Display_types_print;;

let printer_option_external_program_command ctx ppf = function
  | s_l -> Og_printer.printer_List Og_printer.printer_String ctx ppf s_l
;;

let printer_option_dimension ctx ppf = function
  | i -> Og_printer.printer_Int ctx ppf i
;;

let printer_option_dimension_management ctx ppf = function
  | d -> printer_dimension_management ctx ppf d
;;

let rec printer_option_refinement_strategy _ctx ppf = function
  | Options_types.Option_Best_in_a_family ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Option_Best_in_a_family"
  | Options_types.Option_Overall_best ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Option_Overall_best"

and printer_option_number_of_cuts ctx ppf = function
  | i -> Og_printer.printer_Int ctx ppf i

and printer_option_basic_refinement_family ctx ppf = function
  | b -> printer_basic_refinement_family ctx ppf b

and printer_option_basic_refinement_families ctx ppf = function
  | o_l ->
    Og_printer.printer_List printer_option_basic_refinement_family
      ctx
      ppf
      o_l
;;

let printer_option_candidate_refinements_selection_criterion_ratio ctx ppf =
  function
  | f -> Og_printer.printer_Float ctx ppf f

and printer_option_candidate_refinements_selection_criterion_maximum_number
  ctx ppf = function
  | i -> Og_printer.printer_Int ctx ppf i
;;

let printer_option_min_relative_gradients_norm ctx ppf = function
  | f -> Og_printer.printer_Float ctx ppf f

and printer_option_min_relative_best_scalar_indicator ctx ppf = function
  | f -> Og_printer.printer_Float ctx ppf f

and printer_option_min_relative_scalar_cost_decrease ctx ppf = function
  | f -> Og_printer.printer_Float ctx ppf f

and printer_option_max_iteration ctx ppf = function
  | i -> Og_printer.printer_Int ctx ppf i
;;

let printer_option_out_formatter ctx ppf = function
  | Options_types.Option_Std_out ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Option_Std_out"
  | Options_types.Option_Std_err ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Option_Std_err"
  | Options_types.Option_Out_file u_t ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Option_Out_file";
    Ufilename_print.printer_t Og_precedence.Ck_argument ppf u_t;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
;;

let printer_option_report_allocated_memory ctx ppf = function
  | b -> Og_printer.printer_Bool ctx ppf b
;;

let printer_option_incremental_mode ctx ppf = function
  | b -> Og_printer.printer_Bool ctx ppf b
;;

let printer_option_graphics_mode ctx ppf = function
  | g -> printer_graphics_mode ctx ppf g
;;

let printer_option_graphics_window_min_width ctx ppf = function
  | i -> Og_printer.printer_Int ctx ppf i

and printer_option_graphics_window_max_width ctx ppf = function
  | i -> Og_printer.printer_Int ctx ppf i

and printer_option_graphics_window_min_height ctx ppf = function
  | i -> Og_printer.printer_Int ctx ppf i

and printer_option_graphics_window_max_height ctx ppf = function
  | i -> Og_printer.printer_Int ctx ppf i
;;

let printer_option_graphics_window_background_color ctx ppf = function
  | c -> printer_color ctx ppf c
;;

let printer_option_graphics_color_scale_name _ctx ppf = function
  | Options_types.Gray ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Gray"
  | Options_types.Cool ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Cool"
  | Options_types.Hot ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Hot"
  | Options_types.Jet ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Jet"
;;

let printer_option_graphics_parameter_min ctx ppf = function
  | f -> Og_printer.printer_Float ctx ppf f

and printer_option_graphics_parameter_max ctx ppf = function
  | f -> Og_printer.printer_Float ctx ppf f
;;

let printer_option_graphics_gradient_amplitude ctx ppf = function
  | f -> Og_printer.printer_Float ctx ppf f
;;

(* Defining top printers. *)

let print_option_external_program_command ppf =
  printer_option_external_program_command Og_precedence.Ck_toplevel ppf
;;

let print_option_dimension ppf =
  printer_option_dimension Og_precedence.Ck_toplevel ppf
;;

let print_option_dimension_management ppf =
  printer_option_dimension_management Og_precedence.Ck_toplevel ppf
;;

let print_option_refinement_strategy ppf =
  printer_option_refinement_strategy Og_precedence.Ck_toplevel ppf

and print_option_number_of_cuts ppf =
  printer_option_number_of_cuts Og_precedence.Ck_toplevel ppf

and print_option_basic_refinement_family ppf =
  printer_option_basic_refinement_family Og_precedence.Ck_toplevel ppf

and print_option_basic_refinement_families ppf =
  printer_option_basic_refinement_families Og_precedence.Ck_toplevel ppf
;;

let print_option_candidate_refinements_selection_criterion_ratio ppf =
  printer_option_candidate_refinements_selection_criterion_ratio
    Og_precedence.Ck_toplevel
    ppf

and print_option_candidate_refinements_selection_criterion_maximum_number
  ppf =
  printer_option_candidate_refinements_selection_criterion_maximum_number
    Og_precedence.Ck_toplevel
    ppf
;;

let print_option_min_relative_gradients_norm ppf =
  printer_option_min_relative_gradients_norm Og_precedence.Ck_toplevel ppf

and print_option_min_relative_best_scalar_indicator ppf =
  printer_option_min_relative_best_scalar_indicator
    Og_precedence.Ck_toplevel
    ppf

and print_option_min_relative_scalar_cost_decrease ppf =
  printer_option_min_relative_scalar_cost_decrease
    Og_precedence.Ck_toplevel
    ppf

and print_option_max_iteration ppf =
  printer_option_max_iteration Og_precedence.Ck_toplevel ppf
;;

let print_option_out_formatter ppf =
  printer_option_out_formatter Og_precedence.Ck_toplevel ppf
;;

let print_option_report_allocated_memory ppf =
  printer_option_report_allocated_memory Og_precedence.Ck_toplevel ppf
;;

let print_option_incremental_mode ppf =
  printer_option_incremental_mode Og_precedence.Ck_toplevel ppf
;;

let print_option_graphics_mode ppf =
  printer_option_graphics_mode Og_precedence.Ck_toplevel ppf
;;

let print_option_graphics_window_min_width ppf =
  printer_option_graphics_window_min_width Og_precedence.Ck_toplevel ppf

and print_option_graphics_window_max_width ppf =
  printer_option_graphics_window_max_width Og_precedence.Ck_toplevel ppf

and print_option_graphics_window_min_height ppf =
  printer_option_graphics_window_min_height Og_precedence.Ck_toplevel ppf

and print_option_graphics_window_max_height ppf =
  printer_option_graphics_window_max_height Og_precedence.Ck_toplevel ppf
;;

let print_option_graphics_window_background_color ppf =
  printer_option_graphics_window_background_color
    Og_precedence.Ck_toplevel
    ppf
;;

let print_option_graphics_color_scale_name ppf =
  printer_option_graphics_color_scale_name Og_precedence.Ck_toplevel ppf
;;

let print_option_graphics_parameter_min ppf =
  printer_option_graphics_parameter_min Og_precedence.Ck_toplevel ppf

and print_option_graphics_parameter_max ppf =
  printer_option_graphics_parameter_max Og_precedence.Ck_toplevel ppf
;;

let print_option_graphics_gradient_amplitude ppf =
  printer_option_graphics_gradient_amplitude Og_precedence.Ck_toplevel ppf
;;

(* End of OCamlGen, 4.2+dev0 generated code. *)
