(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: parameter.mli,v 1.9 2017/05/31 17:27:44 fclement Exp $ *)

open Refinement_types;;

val same_geometry : 'a parameter -> 'a parameter -> bool;;

(** {6 Converters} *)

val sub :
  ('a -> 'a -> 'a) -> 'a parameter -> 'a sub_parameter -> 'a sub_parameter
(** [sub_parameter f gim sgim] returns the sub-parameter with the domain of
 [sgim], but with images obtained by the application of [f] to the images of
 [gim] and [sgim].
 [gim] is supposed to be a full parameter, and the domain of [sgim] is supposed
 to be a subset of the domain of [gim]. *)
;;

(** {6 Iterators} *)

val map : ('a -> 'b) -> 'a parameter -> 'b parameter;;
val fold : ('accu -> 'a -> 'accu) -> 'accu -> 'a parameter -> 'accu;;

(** {6 Norms} *)

val norm_sup : ('a -> float) -> 'a parameter -> float;;

(** {6 Domain operations} *)

val bounds_of_domain : Umatrix_nonempty_set.t -> bounds;;
