(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: state_spec.mli,v 1.26 2017/11/07 20:57:41 fclement Exp $ *)

(** {3 State specification} *)

open Refinement_types;;

module type MAKER =
  functor (Scal : Scalar_spec.SCALAR) ->
  functor (Comps : Components_spec.COMPONENTS) ->
sig
  module Bb : module type of (Black_box.Make (Scal) (Comps));;

  type scalar = Scal.t;;
  type 'a components = 'a Comps.t;;
  type t = Bb.state;;
  type selected_refinement = Bb.selected_refinement;;
  type constants = Constants_types.t;;

  type forced_zonations = scalar forced_zonation components;;

  val same : t -> t -> bool;;
  val all_equal_coarse_parameters :
    Bb.partitions list -> Bb.selected_refinement list -> bool
  ;;

  val printer : t Og_types.printer;;
  val print : t Og_types.top_printer;;
  val printer_selected_refinement : selected_refinement Og_types.printer;;
  val print_selected_refinement : selected_refinement Og_types.top_printer;;
  val printer_constants : constants Og_types.printer;;
  val print_constants : constants Og_types.top_printer;;

  val init : unit -> t;;
  val init_constants : t -> constants;;

  val vector_first_order_indicators_overall_best :
    forced_zonations -> forced_zonations
  ;;
  val vector_first_order_indicators_best_in_a_family :
    forced_zonations -> forced_zonations
  ;;

  val max_first_order_indicator :
    forced_zonations -> scalar first_order_indicator
  ;;

  val select_n_best_cc :
    forced_zonations -> (scalar first_order_indicator -> bool) ->
    Unatural.t -> Bb.partitions list
  ;;

  val final : t -> unit;;
end
;;
