(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: worker_types.mli,v 1.1 2017/11/07 23:48:44 fclement Exp $ *)

(** {3 Generic part of workers for the inversion platform} *)

open Wire_types;;
open Wrapper_types;;

(** {6 Tasks} *)

type init_engine_in = init_in
and init_engine_out = size * masked_partition * partition * coarse_parameter

and grad_engine_in = grad_in
and grad_engine_out = grad_out

and optim_engine_in = optim_in
and optim_engine_out = optim_out

and final_engine_in = final_in
and final_engine_out = final_out
;;

type init_engine = Pio.spf -> init_engine_in -> init_engine_out
and grad_engine = Pio.spf -> grad_engine_in -> grad_engine_out
and optim_engine = Pio.spf -> optim_engine_in -> optim_engine_out
and final_engine = Pio.spf -> final_engine_in -> final_engine_out
;;

type task_engines =
  {
   init_engine : init_engine;
   grad_engine : grad_engine;
   optim_engine : optim_engine;
   final_engine : final_engine;
  }
;;

type tasks =
  {
   init : (init_in, init_out) Pio.task;
   grad : (grad_in, grad_out) Pio.task;
   optim : (optim_in, optim_out) Pio.task;
   final : (final_in, final_out) Pio.task;
  }
;;
