CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                              Ref-indic                              C
C                                                                     C
C                        Refinement indicators                        C
C                                                                     C
C                       EPI Serena, INRIA Paris                       C
C                                                                     C
C  Copyright (c) 2013-2017 INRIA.                                     C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C     $Id: optim.f,v 1.10 2017/05/31 17:27:50 fclement Exp $

      subroutine optim(
     +     ncells, projS, nzonesS, paramS, projT, nzonesT, paramT,
     +     cout, len_sc, sc, ier, emsg_length, error_msg)
      
      implicit none
      
      integer OUTPUT_UNIT
      parameter(OUTPUT_UNIT=18)
      
      include 'common.fh'
      include 'optim_common.fh'
      
C     Input arguments.
      integer ncells, nzonesS, nzonesT
      integer projS(np_max), projT(np_max)
      
C     Input/output argument.
      real*8 paramS(np_max), paramT(np_max)
      
C     Output argument.
      character sc*(*), error_msg*80
      integer len_sc, ier, emsg_length
      real*8 cout
      
C     Variables locales.
      integer i, j, izs(2*np_max+3), izs_ptr, nparam, nbd(2*np_max)
      real rzs(1), dzs(1)
      real*8 param(2*np_max), gdcout(2*np_max)
      real*8 lbd(2*np_max), ubd(2*np_max)
      
      external simul
      
C     Empaquetage de ncells, projS, nzonesS, projT, nzonesT
      izs_ptr = 1
      izs(izs_ptr) = ncells
      izs_ptr = izs_ptr + 1
      do i = 1,ncells
         izs(izs_ptr) = projS(i)
         izs_ptr = izs_ptr + 1
      enddo
      izs(izs_ptr) = nzonesS
      izs_ptr = izs_ptr + 1
      do i = 1,ncells
         izs(izs_ptr) = projT(i)
         izs_ptr = izs_ptr + 1
      enddo
      izs(izs_ptr) = nzonesT
      rzs(1) = 0.
      dzs(1) = 0d0
      
      j = 0
      do i = 1,nzonesS
         j = j + 1
         param(j) = paramS(i)
      enddo
      do i = 1,nzonesT
         j = j + 1
         param(j) = paramT(i)
      enddo
      nparam = j
      
      if (idb .ne. 0) then
         write(OUTPUT_UNIT, *) 'In optim, before first simul'
      endif
      call simul(4, nparam, param, cout, gdcout, izs, rzs, dzs)
      write(OUTPUT_UNIT,*) 'cost function before optim', cout
      
      j = 0
      do i = 1, nzonesS
         j = j + 1
         nbd(j) = nbd_init
         lbd(j) = lS_init
         ubd(j) = uS_init
      enddo
      do i = 1, nzonesT
         j = j + 1
         nbd(j) = nbd_init
         lbd(j) = lT_init
         ubd(j) = uT_init
      enddo
      
      task = 'START'
      
 10   continue
      
      call setulb(nparam,m,param,lbd,ubd,nbd,cout,gdcout,
     +     factr,pgtol,wa,iwa,task,iprint,io,
     +     csave,lsave,isave,dsave)
      
      if (task(1:2) .eq. 'FG') then
         
         call simul(4, nparam, param, cout, gdcout, izs, rzs, dzs)
         goto 10
         
      else if (task(1:5) .eq. 'NEW_X') then
         
         if (isave(34) .gt. nsim_max) then
            task='STOP: TOTAL NO. of f AND g EVALUATIONS EXCEEDS LIMIT'
         else if (isave(30) .gt. iter_max) then
            task='STOP: TOTAL NO. of ITERATIONS EXCEEDS LIMIT'
         endif
         goto 10
         
      else

         write(OUTPUT_UNIT, *) task
         goto 20
         
      endif
      
 20   continue
      
      if (task(1:4) .eq. 'CONV') then
         len_sc = 14
         sc = 'OS_Convergence'
      else if (task(1:43) .eq.
     +        'STOP: TOTAL NO. of ITERATIONS EXCEEDS LIMIT') then
         len_sc = 22
         sc = 'OS_Too_many_iterations'
      else if (task(1:52) .eq.
     +        'STOP: TOTAL NO. of f AND g EVALUATIONS EXCEEDS LIMIT')
     +        then
         len_sc = 23
         sc = 'OS_Too_many_simulations'
      else if (task(1:28) .eq. 'WARNING: XTOL TEST SATISFIED') then
         len_sc = 13
         sc = 'OS_Stagnation'
      else
         len_sc = 22
         sc = 'OS_Degenerated_problem'
      endif
      
      call simul(1, nparam, param, cout, gdcout, izs, rzs, dzs)
      write(OUTPUT_UNIT,*) 'cost function after optim', cout
      
      j = 0
      do i = 1,nzonesS
         j = j + 1
         paramS(i) = param(j)
      enddo
      do i = 1,nzonesT
         j = j + 1
         paramT(i) = param(j)
      enddo
      
      return
      end
