CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                              Ref-indic                              C
C                                                                     C
C                        Refinement indicators                        C
C                                                                     C
C                       EPI Serena, INRIA Paris                       C
C                                                                     C
C  Copyright (c) 2013-2017 INRIA.                                     C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C     $Id: main.fpp,v 1.11 2017/11/05 19:55:56 fclement Exp $

#include "worker_constants.h"
#include "model_constants.h"

      program main
C     Main program.

      implicit none

C     External routines.
      external init_engine, grad_engine, optim_engine, final_engine

      call worker_main_body
     +     (init_engine, grad_engine, optim_engine, final_engine)

      end


      subroutine init_engine(mp1, pp1, lens1, m1)
C     Initialization.

      implicit none

      include 'worker_error_definition.fh'
      include 'worker_init_definition.fh'

C     Output arguments.
      integer mp1(WORKER_MAX_MSIZE, WORKER_MAX_DIMENSION)
      integer pp1(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)
      integer lens1(WORKER_MAX_DIMENSION)
      FLOAT m1(WORKER_MAX_ZONE_NUMBER)

C     Local variables.
      character sbr*11, msg*MAX_ERROR_MSG_LENGTH
      integer mp(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)
      integer s_len, len, dim_loc, i, j

      s_len = 11
      sbr = 'init_engine'

      if (.not. ((dim .ge. 1) .or. (dim .le. 4))) then
         call worker_error_unbound_dimension(s_len, sbr, dim)
         return
      endif

      call init(dim_loc, size_x, size_y, msize, size, ier, len, msg)
      if (ier .ne. 0) then
         call worker_error_init_engine(s_len, sbr, len, msg)
         return
      endif

      if (dim_loc .ne. dim) then
         len = 26 + 1 + 11
         write(msg, 1001) dim
         call worker_error_init_engine(s_len, sbr, len, msg)
         return
      endif

 1001 format('input image does not have ', i1, ' channel(s)')

C     Checkerboard optional mask.
      call identity_checkerboard_masked_partition(mp)

      do j = 1, dim
         do i = 1, msize
            mp1(i, j) = mp(i, j)
         enddo
      enddo

C     One zone per channel.
      call worker_trivial_init(WORKER_ZERO, pp1, lens1, m1)

      return
      end


      subroutine grad_engine(p, g)
C     Gradient computation.

      implicit none

      include 'worker_error_definition.fh'
      include 'worker_init_definition.fh'

C     Input argument.
      FLOAT p(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)

C     Output argument.
      FLOAT g(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)

C     Local variables.
      character sbr*11, msg*MAX_ERROR_MSG_LENGTH
      integer s_len, len, i, j
      real*8 fine_p(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)
      real*8 fine_g(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)

      s_len = 11
      sbr = 'grad_engine'

      do j = 1, dim
         do i = 1, size
            fine_p(i, j) = p(i, j)
         enddo
      enddo

      call gradient(dim, size, fine_p, fine_g, ier, len, msg)
      if (ier .ne. 0) then
         call worker_error_grad_engine(s_len, sbr, len, msg)
         return
      endif

      do j = 1, dim
         do i = 1, size
            g(i, j) = fine_g(i, j)
         enddo
      enddo

      return
      end


      subroutine optim_engine(pp, lens, m, len_sc, sc, cost)
C     Optimal coarse parameter computation.

      implicit none

      include 'worker_error_definition.fh'
      include 'worker_init_definition.fh'

C     Input arguments.
      integer pp(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)
      integer lens(WORKER_MAX_DIMENSION)

C     Input/output argument.
      FLOAT m(WORKER_MAX_ZONE_NUMBER)

C     Output argument.
      character sc*(*)
      integer len_sc
      FLOAT cost

C     Local variables.
      character sbr*12, msg*MAX_ERROR_MSG_LENGTH
      integer s_len, len, i, j, k
      integer partition(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)
      integer cp_len(MAX_CHANNEL_NUMBER)
      real*8 coarse_p(MAX_ZONE_NUMBER), jj

      s_len = 12
      sbr = 'optim_engine'

      do j = 1, dim
         do i = 1, size
            partition(i, j) = pp(i, j)
         enddo
         cp_len(j) = lens(j)
      enddo

      do k = 1, zone_number
         coarse_p(k) = m(k)
      enddo

      call optim(dim, size, zone_number,
     +     partition, cp_len, coarse_p, jj, ier, len, msg)
      if (ier .ne. 0) then
         call worker_error_optim_engine(s_len, sbr, len, msg)
         return
      endif

      len_sc = 14
      sc(1:len_sc) = 'OS_Convergence'

      do k = 1, zone_number
         m(k) = coarse_p(k)
      enddo

      cost = jj

      return
      end


      subroutine final_engine(p)
C     Final(ization) engine.

      implicit none

      include 'worker_error_definition.fh'
      include 'worker_init_definition.fh'

C     Input argument.
      FLOAT p(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)

C     Local variables.
      character sbr*12, msg*MAX_ERROR_MSG_LENGTH
      integer s_len, len, i, j
      real*8 fine_p(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)

      s_len = 12
      sbr = 'final_engine'

      do j = 1, dim
         do i = 1, size
            fine_p(i, j) = p(i, j)
         enddo
      enddo

      call final(dim, size, fine_p, ier, len, msg)
      if (ier .ne. 0) then
         call worker_error_final_engine(s_len, sbr, len, msg)
         return
      endif

      return
      end
